import warnings
from unittest import TestCase

from pydantic import BaseModel as PydanticBaseModel, Field
from testfixtures import Comparison as C, SequenceComparison as S, compare

from fred_types.common import BaseId, BaseModel
from fred_types.exceptions import ExtraArgumentWarning


class BaseModelTest(TestCase):
    def test_warning(self):
        class TestModel(BaseModel):
            name: str

        with warnings.catch_warnings(record=True) as captured:
            warnings.simplefilter("always")

            TestModel(name="Kryten", id="2X4B", ship="Red Dwarf")  # type: ignore[call-arg]

        message = "Unexpected extra argument %s for model fred_types.tests.test_common.TestModel"
        compare(
            S(
                C(ExtraArgumentWarning(message % "id")),
                C(ExtraArgumentWarning(message % "ship")),
                ordered=False,
            ),
            actual=[c.message for c in captured],
        )

    def test_no_dict(self):
        class TestModel(BaseModel):
            name: str

        with warnings.catch_warnings(record=True) as captured:
            warnings.simplefilter("always")

            TestModel._warn_extra(TestModel.model_construct(name="Kryten"))  # type: ignore[operator]

        self.assertEqual(captured, [])

    def test_alias(self):
        class TestModel(BaseModel):
            name: str = Field(alias="nickname")

        with warnings.catch_warnings(record=True) as captured:
            warnings.simplefilter("always")

            TestModel(nickname="Kryten")  # type: ignore[call-arg]

        self.assertEqual(len(captured), 0)


class BaseIdTest(TestCase):
    def test_validation(self):
        class Model(PydanticBaseModel):
            uid: BaseId

        model = Model(uid="007")  # type: ignore[arg-type]

        self.assertEqual(model.uid, BaseId("007"))
        self.assertTrue(isinstance(model.uid, BaseId))

    def test_repr(self):
        self.assertEqual(repr(BaseId("007")), "BaseId('007')")
